package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerInput;
import pps.cloud.prediction.service.data.storage_predicted_power_data.DynamicQueryStoragePredictedPowerInput;
import pps.cloud.space.service.ISpaceOptimizeLongCloudService;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.utils.BaseUtils;
import pps.core.space.entity.*;
import pps.core.space.mapper.SpaceOptimizeLongPeriodMapper;
import pps.core.space.mapper.SpaceOptimizeViewMapper;
import pps.core.space.service.data.SpaceOptimizeLineRelation;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;
import xstartup.helper.XTransactionHelper;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 中期间开优化Cloud模块
 *
 * @author ZWT
 * @date 2023/09/13 14:14
 */
@XService
public class SpaceOptimizeLongCloudServiceImpl extends SpaceOptimizeBaseService implements ISpaceOptimizeLongCloudService {

    /**
     * 中期间开优化Cloud模块--定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult optimizeLongJob(XContext context) {
        //取当前时间
        DateTime date = DateUtil.beginOfDay(DateTime.now());
        //取生效中的基础间开
        List<SpaceInstitutionDetailEnt> detailEntList = super.getEffectiveSpaceInstitution(context, date.toString(BusinessConstant.DATE_FORMAT_DAY));
        SpaceOptimizeLineRelation relation = super.getOptimizeLineRelation(context, detailEntList);
        //月份要加一
        int monthIndex = date.month();
        int monthNum = monthIndex + 1;
        List<SpaceOptimizePeriodDTO> periodDTOList = new ArrayList<>(32);
        List<SpaceOptimizeWellheadDTO> wellheadDTOList = new ArrayList<>(64);
        List<SpaceOptimizeDurationDTO> durationDTOList = new ArrayList<>(128);
        List<SpaceOptimizeDurationDTO> unOptimizeDurationList = new ArrayList<>(128);
        //优化
        for (SpaceInstitutionDetailEnt detail : detailEntList) {
            //查最新间开历史
            SpaceOptimizeLongPeriodMapper periodMapper = context.getBean(SpaceOptimizeLongPeriodMapper.class);
            SpaceOptimizeLongPeriodEnt longPeriodEnt = periodMapper.selectOne(new QueryWrapper<SpaceOptimizeLongPeriodEnt>()
                    .select("institution_id",
                            "MAX( optimize_deadline ) AS optimize_deadline")
                    .lambda()
                    .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                    .eq(SpaceOptimizeLongPeriodEnt::getInstitutionId, detail.getId())
            );
            if (ObjectUtil.isNotNull(longPeriodEnt)) {
                Date optimizeDead = longPeriodEnt.getOptimizeDeadline();
                if (DateUtil.compare(date, optimizeDead) == -1) {
                    continue;
                }
            }
            Date institutionEndDate = detail.getInstitutionEndDate();
            DateTime optimizeDeadline = DateUtil.offsetDay(date, 10);
            if (optimizeDeadline.compareTo(institutionEndDate) > 0) {
                optimizeDeadline = DateUtil.endOfDay(institutionEndDate);
            }
            //创建记录
            String periodId = super.createOptimizePeriod(periodDTOList, detail.getId(), detail.getLineId(), BaseUtils.getExecutionCycleForCalibration(date, optimizeDeadline), optimizeDeadline);
            switch (detail.getGridTypeKey()) {
                //并网型优化
                case "1":
                    super.gridConnectedOptimization(context, wellheadDTOList, durationDTOList, unOptimizeDurationList,
                            relation.getWellheadsByInstitutionIdIdMap().get(detail.getId()), detail, monthNum, periodId,
                            DynamicQueryPlantPredictedPowerInput.builder()
                                    .plantIds(relation.getPlantIdsByLineIdMap().get(detail.getLineId()))
                                    .startTime(date.toString(BusinessConstant.DATE_FORMAT_DAY))
                                    .endTime(optimizeDeadline.toString(BusinessConstant.DATE_FORMAT_DAY))
                                    .dateType(BusinessConstant.ZERO)
                                    .build());
                    break;
                //离网型优化
                case "0":
                    super.offGridOptimization(context, wellheadDTOList, durationDTOList, unOptimizeDurationList, relation.getWellheadsByInstitutionIdIdMap().get(detail.getId()),
                            relation.getStorageAvgMap(), detail, periodId,
                            DynamicQueryStoragePredictedPowerInput.builder()
                                    .storageIds(relation.getStorageIdsByLineIdMap().get(detail.getLineId()))
                                    .startTime(DateUtil.offsetDay(date, -10).toString(BusinessConstant.DATE_FORMAT_DAY))
                                    .endTime(DateUtil.offsetDay(optimizeDeadline, -10).toString(BusinessConstant.DATE_FORMAT_DAY))
                                    .build(),
                            DynamicQueryPlantPredictedPowerInput.builder()
                                    .plantIds(relation.getPlantIdsByLineIdMap().get(detail.getLineId()))
                                    .startTime(date.toString(BusinessConstant.DATE_FORMAT_DAY))
                                    .endTime(optimizeDeadline.toString(BusinessConstant.DATE_FORMAT_DAY))
                                    .dateType(BusinessConstant.ZERO)
                                    .build());
                    break;
                default:
                    //电网类型不存在
            }
        }
        //开启事务
        return XTransactionHelper.begin(context, () -> {
            int size;
            SpaceOptimizeViewMapper optimizeViewMapper = context.getBean(SpaceOptimizeViewMapper.class);
            if (CollUtil.isNotEmpty(periodDTOList)) {
                size = periodDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizePeriodDTO>> subList = BaseUtils.getSubList(periodDTOList);
                    subList.forEach(optimizeViewMapper::longPeriodBatchInsertList);
                } else {
                    optimizeViewMapper.longPeriodBatchInsertList(periodDTOList);
                }
            }
            if (CollUtil.isNotEmpty(wellheadDTOList)) {
                size = wellheadDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeWellheadDTO>> subList = BaseUtils.getSubList(wellheadDTOList);
                    subList.forEach(optimizeViewMapper::longWellheadBatchInsertList);
                } else {
                    optimizeViewMapper.longWellheadBatchInsertList(wellheadDTOList);
                }
            }
            if (CollUtil.isNotEmpty(unOptimizeDurationList)) {
                size = unOptimizeDurationList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeDurationDTO>> subList = BaseUtils.getSubList(unOptimizeDurationList);
                    subList.forEach(optimizeViewMapper::longDurationBatchInsertList);
                } else {
                    optimizeViewMapper.longDurationBatchInsertList(unOptimizeDurationList);
                }
            }
            if (CollUtil.isNotEmpty(durationDTOList)) {
                size = durationDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeDurationDTO>> subList = BaseUtils.getSubList(durationDTOList);
                    subList.forEach(optimizeViewMapper::longDurationBatchInsertList);
                } else {
                    optimizeViewMapper.longDurationBatchInsertList(durationDTOList);
                }
            }
            return XServiceResult.OK;
        });
    }
}